# encoding=utf-8

from django.test import TestCase
from django.test import Client
from django.core.urlresolvers import reverse
from django.conf import settings
from django.core.cache import cache
from django.contrib.auth.tests.utils import skipIfCustomUser
from django.contrib.auth.models import User
from django.test.utils import override_settings

import hashlib
import time
from mock import Mock
import simplejson as json
import random
from hashlib import sha1

from goldencage import views
from goldencage import config
from goldencage.models import task_done
from goldencage.models import appwalllog_done
from goldencage.models import payment_done
from goldencage.models import apply_coupon
from goldencage.models import AppWallLog
from goldencage.models import Charge
from goldencage.models import ChargePlan
from goldencage.models import Task
from goldencage.models import Order
from goldencage.models import Coupon
from goldencage.models import Exchange


@skipIfCustomUser
class CouponModelTest(TestCase):

    """测试优惠券生成与验证。
    生成：
    - 如果有次数限制
    如果完成了的次数已达到限制，返回空
    - 无次数限制或次数未到。
    有未使用券，则直接使用
    无未使用券，生成新的。

    验证：
    - 无券，返回False
    - 有券，但已完成，返回False
    - 有券，未完成，返回True，同时发出signal
    """

    def test_generate_normal(self):

        coupon = Coupon(name='test', cost=10, limit=1,
                        key='test')
        coupon.save()
        user = User.objects.create_user('jeff', 'jeff@toraysoft.com', '123')
        exc = coupon.generate(user)

        self.assertIsNotNone(exc)

    def test_generate_duplidate(self):
        coupon = Coupon(name='test', cost=10, limit=1,
                        key='test')

        coupon.save()
        user = User.objects.create_user('jeff', 'jeff@toraysoft.com', '123')
        vera = User.objects.create_user('vera', 'jeff@toraysoft.com', '123')
        exc = Exchange(coupon=coupon, user=user, cost=10, status='WAITING',
                       exchange_code='1233')
        exc.save()
        e = coupon.generate(vera, default=1233)
        self.assertNotEquals(e.exchange_code, '1233')

    def test_generate_limit(self):
        coupon = Coupon(name='test', cost=10, limit=1,
                        key='test')
        coupon.save()
        user = User.objects.create_user('jeff', 'jeff@toraysoft.com', '123')

        exc = Exchange(coupon=coupon, user=user, cost=10, status='DONE',
                       exchange_code='1233')
        exc.save()

        e = coupon.generate(user)
        self.assertIsNone(e)

    def test_generate_reuse(self):
        coupon = Coupon(name='test', cost=10, limit=1,
                        key='test')
        coupon.save()
        user = User.objects.create_user('jeff', 'jeff@toraysoft.com', '123')

        exc = Exchange(coupon=coupon, user=user, cost=10, status='WAITING',
                       exchange_code='1233')
        exc.save()

        e = coupon.generate(user)
        self.assertIsNotNone(e)
        self.assertEqual('1233', e.exchange_code)

    def test_valid_notfound(self):
        coupon = Coupon(name='test', cost=10, limit=1,
                        key='test')
        coupon.save()

        result = coupon.validate('1233')
        self.assertFalse(result)

    def test_valid_duplicate(self):
        coupon = Coupon(name='test', cost=10, limit=1,
                        key='test')
        coupon.save()
        user = User.objects.create_user('jeff', 'jeff@toraysoft.com', '123')

        exc = Exchange(coupon=coupon, user=user, cost=10, status='DONE',
                       exchange_code='1233')
        exc.save()

        result = coupon.validate('1233')
        self.assertFalse(result)

    def test_valid_normal(self):
        coupon = Coupon(name='test', cost=20, limit=1,
                        key='test')
        coupon.save()
        user = User.objects.create_user('jeff', 'jeff@toraysoft.com', '123')

        exc = Exchange(coupon=coupon, user=user, cost=10, status='WAITING',
                       exchange_code='1233')
        exc.save()

        apply_coupon.send = Mock()

        result = coupon.validate('1233')
        self.assertEqual(result.status, 'DONE')
        apply_coupon.send.assert_called_with(sender=Coupon, instance=coupon,
                                             cost=20, user=user)


class OrderModelTest(TestCase):

    def test_get_real_id_without_prefix(self):
        self.assertEqual(999999999, Order.get_real_id(999999999))

    @override_settings(GOLDENCAGE_ORDER_ID_PREFIX=9)
    def test_get_real_id_prefix(self):
        self.assertEqual(999, Order.get_real_id(900000999))

    def test_get_order_id(self):
        order = Order()
        order.id = 100
        gid = order.gen_order_id()
        self.assertEqual(100, gid)

    @override_settings(GOLDENCAGE_ORDER_ID_PREFIX=9)
    def test_gen_order_id_prefix(self):
        order = Order()
        order.id = 100
        gid = order.gen_order_id()
        self.assertEqual(900000100, gid)

    @override_settings(GOLDENCAGE_ORDER_ID_PREFIX=9)
    def test_gen_order_id_prefix_repeat(self):
        order = Order()
        order.id = 999
        gid = order.gen_order_id()
        self.assertEqual(900000999, gid)


@skipIfCustomUser
class TaskModelTest(TestCase):

    def test_make_log_random(self):
        # 测试随机金币
        user = User.objects.create_user('jeff', 'jeff@toraysoft.com', '123')
        task = Task(name='check in', key='check_in',
                    cost=10, cost_max=100)
        task.save()

        log = task.make_log(user)
        assert log.cost >= 10 and log.cost <= 100

    def test_make_log_infinity(self):
        # 测试随机金币
        user = User.objects.create_user('jeff', 'jeff@toraysoft.com', '123')
        task = Task(name='check in', key='check_in',
                    cost=10)
        task.save()

        log = task.make_log(user)
        self.assertEqual(10, log.cost)
        log = task.make_log(user)
        self.assertEqual(10, log.cost)


@skipIfCustomUser
class AppWallCallbackTest(TestCase):

    def test_waps_callback(self):
        user = User.objects.create_user('jeff', 'jeff@toraysoft.com', '123')
        data = {'adv_id': '1', 'app_id': 'theme',
                'key': user.pk, 'udid': 'myudid',
                'openudid': 'myopenid', 'bill': '2.0',
                'points': 200, 'ad_name': 'music talk'
                }
        appwalllog_done.send = Mock()
        c = Client()
        rsp = c.get(reverse('wall_cb', args=['waps']), data)
        self.assertEqual(rsp.status_code, 200)
        appwalllog_done.send.assert_called_with(cost=200, user=user,
                                                sender=AppWallLog)

    def test_waps_callback_duplicate(self):
        user = User.objects.create_user('jeff', 'jeff@toraysoft.com', '123')
        data = {'adv_id': '1', 'app_id': 'theme',
                'key': user.pk, 'udid': 'myudid',
                'openudid': 'myopenid', 'bill': '2.0',
                'points': 200, 'ad_name': 'music talk'
                }
        c = Client()
        rsp = c.get(reverse('wall_cb', args=['waps']), data)
        self.assertEqual(rsp.status_code, 200)
        dt = json.loads(rsp.content)
        self.assertTrue(dt['success'])

        rsp = c.get(reverse('wall_cb', args=['waps']), data)
        self.assertEqual(rsp.status_code, 200)
        dt = json.loads(rsp.content)
        self.assertFalse(dt['success'])

    def test_waps_callback_invalid_ip(self):
        c = Client(REMOTE_ADDR='192.168.0.1')
        rsp = c.get(reverse('wall_cb', args=['waps']))
        self.assertEqual(rsp.status_code, 405)

    def create_youmi_ios_data(self, user):
        ts = int(time.time())
        return {'order': 'NO.1', 'app': 'my appid',
                'adid': '1', 'user': user.pk,
                'device': 'mydevice', 'chn': 0,
                'price': '4.9', 'points': 90,
                'time': ts, 'sig': 'xdref', 'ad': 'musictalk'
                }

    def test_youmi_ios_callback(self):
        user = User.objects.create_user('jeff', 'jeff@toraysoft.com', '123')
        data = self.create_youmi_ios_data(user)
        keys = data.keys()
        keys.sort()
        appwalllog_done.send = Mock()
        src = ''.join(['%s=%s' % (k, unicode(data[k]).encode('utf-8'))
                       for k in keys])
        src += settings.YOUMI_CALLBACK_SECRET
        md5 = hashlib.md5()
        md5.update(src.encode('utf-8'))
        sign = md5.hexdigest()
        data['sign'] = sign
        c = Client()
        rsp = c.get(reverse('wall_cb', args=['youmi_ios']), data)
        self.assertEqual(rsp.status_code, 200)
        appwalllog_done.send.assert_called_with(sender=AppWallLog,
                                                cost=90, user=user)

    def test_youmi_ios_missing_sign(self):
        c = Client()
        rsp = c.get(reverse('wall_cb', args=['youmi_ios']))
        self.assertEqual(rsp.status_code, 403)

    def test_youmi_ios_invalidate_sign(self):
        user = User.objects.create_user('jeff', 'jeff@toraysoft.com', '123')
        data = self.create_youmi_ios_data(user)
        data['sign'] = 'not a valid sign'
        c = Client()
        rsp = c.get(reverse('wall_cb', args=['youmi_ios']), data)
        self.assertEqual(rsp.status_code, 403)

    def test_youmi_ios_duplicate(self):
        """同一个订单提交两次"""
        user = User.objects.create_user('jeff', 'jeff@toraysoft.com', '123')
        data = self.create_youmi_ios_data(user)
        keys = data.keys()
        keys.sort()

        src = ''.join(['%s=%s' % (k, unicode(data[k]).encode('utf-8'))
                       for k in keys])
        src += settings.YOUMI_CALLBACK_SECRET
        md5 = hashlib.md5()
        md5.update(src.encode('utf-8'))
        sign = md5.hexdigest()
        data['sign'] = sign
        c = Client()
        rsp = c.get(reverse('wall_cb', args=['youmi_ios']), data)
        self.assertEqual(rsp.status_code, 200)
        # user = User.custom_objects.get(name=user.name)
        # self.assertEqual(user.balance, 90)

        rsp = c.get(reverse('wall_cb', args=['youmi_ios']), data)
        self.assertEqual(rsp.status_code, 403)

    def create_dianjoy_adr_data(self, user):
        ts = int(time.time())
        return {'snuid': user.pk, 'device_id': 'my device',
                'app_id': 'helper', 'currency': 100,
                'app_ratio': 1, 'time_stamp': ts,
                'ad_name': '医生', 'pack_name': 'com.toraysoft.music',
                'trade_type': 1,
                }

    def test_dianjoy_adr_invalid_token(self):
        """点乐：无效token
        """
        user = User.objects.create_user('jeff', 'jeff@toraysoft.com', '123')
        data = self.create_dianjoy_adr_data(user)
        data['token'] = 'not a valid token'
        c = Client()
        rsp = c.get(reverse('wall_cb', args=['dianjoy_adr']), data)
        self.assertEqual(rsp.status_code, 403)

    def test_dianjoy_adr_success(self):
        """点乐，有效
        """
        appwalllog_done = Mock()
        user = User.objects.create_user('jeff', 'jeff@toraysoft.com', '123')
        data = self.create_dianjoy_adr_data(user)
        src = str(data['time_stamp']) + \
            settings.GOLDENCAGE_DIANJOY_ANDROID_SECRET
        md5 = hashlib.md5()
        md5.update(src.encode('utf-8'))
        sign = md5.hexdigest()
        data['token'] = sign
        c = Client()
        rsp = c.get(reverse('wall_cb', args=['dianjoy_adr']), data)
        self.assertEqual(rsp.status_code, 200)
        self.assertEqual(rsp.content, '200')
        appwalllog_done.assert_called()

        appwalllog_done = Mock()
        rsp = c.get(reverse('wall_cb', args=['dianjoy_adr']), data)
        self.assertEqual(rsp.status_code, 200)
        self.assertEqual(rsp.content, 'OK, But duplicate item')
        self.assertFalse(appwalllog_done.called)

    def create_domob_data(self, user):
        ts = int(time.time())
        return {'user': user.pk, 'orderid': 'orderid',
                'pubid': 'pubid', 'ad': 'ad',
                'adid': 112, 'device': 'device',
                'channel': 1, 'price': '11',
                'point': 10, 'ts': ts, 'pkg': 'pkg', 'action': 1,
                'action_name': '签到-1',
                }

    def test_domob_adr(self):
        appwalllog_done = Mock()
        user = User.objects.create_user('jeff', 'jeff@toraysoft.com', '123')
        params = self.create_domob_data(user)
        param_list = sorted(params.iteritems(), key=lambda d: d[0])
        sign = ''
        for param in param_list:
            sign += (str(param[0]) + '=' + str(param[1]))
        sign += str(settings.GOLDENCAGE_DOMOB_PRIVATE_KEY_ANDROID)
        m = hashlib.md5()
        m.update(sign)
        sign = m.hexdigest()
        params['sign'] = sign
        c = Client()
        rsp = c.get(reverse('wall_cb', args=['domob_adr']), params)
        self.assertEqual(rsp.status_code, 200)
        appwalllog_done.assert_called()

        rsp = c.get(reverse('wall_cb', args=['domob_adr']), params)
        self.assertEqual(rsp.status_code, 200)
        self.assertFalse(appwalllog_done.called)
        self.assertEqual(rsp.content, 'OK, But Duplicated item')

        params['sign'] = 'haha'
        rsp = c.get(reverse('wall_cb', args=['domob_adr']), params)
        self.assertEqual(rsp.status_code, 403)
        self.assertFalse(appwalllog_done.called)

    def test_domob_ios(self):
        appwalllog_done = Mock()
        user = User.objects.create_user('jeff', 'jeff@toraysoft.com', '123')
        params = self.create_domob_data(user)
        param_list = sorted(params.iteritems(), key=lambda d: d[0])
        sign = ''
        for param in param_list:
            sign += (str(param[0]) + '=' + str(param[1]))
        sign += str(settings.GOLDENCAGE_DOMOB_PRIVATE_KEY_IOS)
        m = hashlib.md5()
        m.update(sign)
        sign = m.hexdigest()
        params['sign'] = sign
        c = Client()
        rsp = c.get(reverse('wall_cb', args=['domob_adr']), params)
        self.assertEqual(rsp.status_code, 200)
        appwalllog_done.assert_called()


@skipIfCustomUser
class AlipayCallbackTest(TestCase):

    def setUp(self):
        self.user = User.objects.create_user('jeff',
                                             'jeff@toraysoft.com', '123')
        self.plan = ChargePlan(name=u'plan1', code='plan1',
                               value=3000, cost=750, coupon=50)
        self.plan.save()

    def create_payment_data(self):
        order = Order(user=self.user, plan=self.plan, value=3000)
        order.save()
        self.order = order
        return {'notify_time': '', 'notify_type': 'trade_status_sync',
                'notify_id': 'csdfo834jr', 'sign_type': 'RSA',
                'sign': 'no sign this time',
                'out_trade_no': order.pk, 'subject': u'多啦A梦',
                'payment_type': 1, 'trade_no': '2014112323e',
                'trade_status': 'TRADE_FINISHED', 'seller_id': '2088xx',
                'seller_email': 'toraysoft@gmail.com', 'buyer_id': '2088yy',
                'buyer_email': 'bbmyth@gmail.com', 'total_fee': 30,
                'quantity': 1, 'price': 30, 'body': u'不错的叮当主题哦',
                'gmt_create': '', 'gmt_payment': '',
                'is_total_fee_adjust': 'N', 'use_coupon': 'N', 'discount': '0'}

    def test_alipay_callback(self):
        # 正常流程, 第一次状态为等待付款，第二次为交易完成
        data = self.create_payment_data()
        c = Client()
        data['trade_status'] = 'WAIT_BUYER_PAY'
        payment_done.send = Mock()
        task_done.send = Mock()
        views.verify_notify_id = Mock(return_value=True)
        views.verify_alipay_signature = Mock(return_value=True)
        cache.set = Mock(return_value=None)

        rsp = c.get(reverse('alipay_cb'), data)
        self.assertEqual('success', rsp.content)
        self.assertEqual(payment_done.send.call_count, 0)
        self.assertEqual(task_done.send.call_count, 0)

        data['trade_status'] = 'TRADE_FINISHED'
        rsp = c.get(reverse('alipay_cb'), data)
        self.assertEqual('success', rsp.content)
        cost = int(round(config.EXCHANGE_RATE * 30))
        payment_done.send.assert_called_with(sender=Charge,
                                             cost=cost, user=self.user,
                                             plan=self.plan, order=self.order)
        task_done.send.assert_called_with(sender=Task, cost=50,
                                          user=self.user)

    def test_alipay_callback_sign_error(self):
        # 签名错误
        data = self.create_payment_data()
        c = Client()
        views.verify_notify_id = Mock(return_value=True)
        views.verify_alipay_signature = Mock(return_value=False)
        rsp = c.get(reverse('alipay_cb'), data)
        self.assertEqual('error', rsp.content)

    def test_alipay_callback_invalidate_request(self):
        # 非来自支付宝的请求
        data = self.create_payment_data()
        c = Client()
        views.verify_notify_id = Mock(return_value=False)
        views.verify_alipay_signature = Mock(return_value=True)
        rsp = c.get(reverse('alipay_cb'), data)
        self.assertEqual('error', rsp.content)

    def test_alipay_notifyid_duplicated(self):
        # 重复收到同一个通知。通知ID同样。
        data = self.create_payment_data()
        views.verify_notify_id = Mock(return_value=True)
        views.verify_alipay_signature = Mock(return_value=True)

        cache.get = Mock(return_value=None)
        cache.set = Mock()
        payment_done.send = Mock()

        c = Client()
        rsp = c.get(reverse('alipay_cb'), data)
        self.assertEqual('success', rsp.content)
        payment_done.send.assert_called_with(sender=Charge, cost=750,
                                             user=self.user,
                                             plan=self.plan, order=self.order)

        cache.get = Mock(return_value='123')
        payment_done.send = Mock()
        rsp = c.get(reverse('alipay_cb'), data)
        self.assertEqual('error', rsp.content)

        self.assertTrue(cache.get.assert_called)
        self.assertEqual(0, payment_done.send.call_count)

    def test_alipay_callback_status_revert(self):
        # 同一个帐单，状态以先后不同的顺序回调。
        data = self.create_payment_data()
        data['trade_status'] = 'TRADE_FINISHED'
        views.verify_notify_id = Mock(return_value=True)
        views.verify_alipay_signature = Mock(return_value=True)
        cache.set = Mock(return_value=None)
        payment_done.send = Mock()
        c = Client()
        rsp = c.get(reverse('alipay_cb'), data)
        self.assertEqual('success', rsp.content)

        self.assertEqual(1, payment_done.send.call_count)

        payment_done.send = Mock()
        data['trade_status'] = 'WAIT_BUYER_PAY'
        data['notify_id'] = 'another_notify'
        rsp = c.get(reverse('alipay_cb'), data)
        self.assertEqual(0, payment_done.send.call_count)
        self.assertEqual('success', rsp.content)
        self.assertEqual(2, cache.set.call_count)

    def test_alipay_callback_duplicated(self):
        # 同一个帐单，相同状态重复发送，将不会充值成功。
        data = self.create_payment_data()
        data['trade_status'] = 'WAIT_BUYER_PAY'

        views.verify_notify_id = Mock(return_value=True)
        views.verify_alipay_signature = Mock(return_value=True)
        cache.set = Mock()
        payment_done.send = Mock()
        c = Client()
        rsp = c.get(reverse('alipay_cb'), data)
        self.assertEqual('success', rsp.content)

        data['notify_id'] = 'another_notify'
        rsp = c.get(reverse('alipay_cb'), data)
        self.assertEqual('error', rsp.content)

        self.assertEqual(1, cache.set.call_count)
        self.assertEqual(0, payment_done.send.call_count)

    def test_signature(self):
        """ 测试之前，要去settings拷贝一个支付宝公钥
            或者不对这个做单元测试
        """
        sign = (u"DoqHII4KFb5QRp5J/bAQPMI/1nJwHf8IcVHDZvvNR5CHCEmAkelExygYooWi"
                "yWchcBd2WHULCNtPKqFEWQALTynzUAkeF64zq9nyq8nzrVulwcKGnu+l"
                "ja6Sg+2EILb3o8RuFcPOL/YAD5y1FxjJBUM33Z+LDcWgb/+eSMDiTQk=")
        params = {
            u"seller_email": u"randotech@126.com",
            u"gmt_close": u"2014-09-02 11:37:03",
            u"sign": sign,
            u"subject": u"资助20元，赠送520金币",
            u"is_total_fee_adjust": u"N",
            u"gmt_create": u"2014-09-02 11:37:02",
            u"out_trade_no": u"117800",
            u"sign_type": u"RSA",
            u"price": u"20.00",
            u"buyer_email": u"mayuze13999087456@126.com",
            u"discount": u"0.00",
            u"trade_status": u"TRADE_FINISHED",
            u"gmt_payment": u"2014-09-02 11:37:03",
            u"trade_no": u"2014090200701660",
            u"seller_id": u"2088311247579029",
            u"use_coupon": u"N",
            u"payment_type": u"1",
            u"total_fee": u"20.00",
            u"notify_time": u"2014-09-02 11:37:41",
            u"buyer_id": u"2088502310925605",
            u"notify_id": u"be431b210180989044cc985639b2a8635c",
            u"notify_type": u"trade_status_sync",
            u"quantity": u"1"
        }
        print 'views %s' % views.verify_alipay_signature
        result = views.verify_alipay_signature('RSA', sign, params)
        self.assertEqual(True, result)


@skipIfCustomUser
class AlipaySignTest(TestCase):

    def setUp(self):
        pass

    def test_alipay_sign(self):
        # 测试用key
        settings.ALIPAY_PRIVATE_KEY = (
            '-----BEGIN RSA PRIVATE KEY-----\n'
            'MIICXAIBAAKBgQCxgCa64qPZ5IKudC+YdEDi2eyLbAtub2h1aBMmHj3hyc1Vdzjh'
            'HyUUt2rgJ7fQAnjNbypzOOWRjAuSsDhB3HfAdle7pJGU5HhVZEpVdNvvdErOMPj1'
            '9IXjTtSc2kBej3E4ETZB0CAbAo6vGzqN8B33NXwxJ6TE3rO/aPAI0SCnUQIDAQAB'
            'AoGAKWPKpDWJI5wHZQqutowVPVC3ueMd30iXQRldrbvLjkTyXoWIe+Y5TVVf1Jku'
            'YZDR/oV3jpqr3X6cjD4PQDxap+D/246GK+a+eDQDLfleb2RtKF1bl/6jqVcbHtnR'
            'kL0MNbYLkuneigVRCetAcGWRxv+BVVP9DYUBjAUq5GZyqAECQQDaFt64w0lj2Nq2'
            'Zb/izenEHX7d5QfsXL3tI1Mhxvzc2CznoTEQgMWgq8ayHd1KUW3KqtZqlrddxYYP'
            'OIAwHIQRAkEA0FsNqYpgU4VlzGzGrWuVq/JDdKYmWOjrk0UbIpKZtIZvvE5S06IV'
            'KJx2fnKh31riLhIJIqoewcaBVmKCV2QvQQJAfAf1su6dloOGH6XOc5bYFAkSVfAj'
            'iXFVMsCcTuF0fcUUBMfPt6sEulP3NOV3LQUSg+iU+RmuP05O5+kiPjp5gQJBALuG'
            'iBhkw+fIM2Q3LuYc43v7svzFIdR55rUIyLBoM9EIAn8AG4oA4nxHvlp2f/yQRuvi'
            'Lbi2VrJfID+Ir/lJ4UECQCgEcFtaNfdZMkQ7Icsw2xynaSJ/osQbpxcOwq4itZ56'
            'xs80ciaAm/uEY7lKiLMmMrjLLD9PBqsrTHa3bMIFaPw='
            '\n-----END RSA PRIVATE KEY-----')

        words = ('partner="2088311247579029"&seller_id="randotech@126.com"&'
                 'out_trade_no="P5IRN0A7B8P1BR7"&subject="珍珠项链"&'
                 'body="[2元包邮]韩版 韩国 流行饰品太阳花小巧雏菊 珍珠项链2M15"&'
                 'total_fee="10.00"&notify_url="http%3A%2F%2Fwwww.xxx.com"&'
                 'service="mobile.securitypay.pay"&_input_charset="utf-8"&'
                 'payment_type="1"&return_url="www.xxx.com"&it_b_pay="1d"&'
                 'show_url="www.xxx.com"')
        data = {'words': words}
        c = Client()
        rsp = c.post(reverse('alipaysign'), data)
        print rsp.content


class WechatTestCase(TestCase):

    def request_content(self, xml):
        cli = Client()
        token = getattr(settings, 'GOLDENCAGE_WECHAT_TOKEN', '')
        timestamp = str(time.time())
        nonce = str(random.randint(1000, 9999))
        sign_ele = [token, timestamp, nonce]
        sign_ele.sort()
        signature = sha1(''.join(sign_ele)).hexdigest()
        params = {'timestamp': timestamp,
                  'nonce': nonce,
                  'signature': signature,
                  'echostr': '234'}
        query_string = '&'.join(['%s=%s' % (k, v) for k, v in params.items()])
        rsp = cli.post('/gc/wechat/?' + query_string,
                       data=xml, content_type='text/xml').content
        return rsp

    def test_success(self):
        """获取礼券成功
        """
        coupon = Coupon(name='test', cost=10, limit=1,
                        key='bb', exchange_style='wechat')

        coupon.save()
        user = User.objects.create_user('jeff', 'jeff@toraysoft.com', '123')

        xml = """<xml>
        <ToUserName><![CDATA[techparty]]></ToUserName>
        <FromUserName><![CDATA[o_BfQjrOWghP2cM0cN7K0kkR54fA]]></FromUserName>
        <CreateTime>1400131860</CreateTime>
        <MsgType><![CDATA[text]]></MsgType>
        <Content><![CDATA[bb1233]]></Content>
        <MsgId>1234567890123456</MsgId>
        </xml>"""
        rsp = self.request_content(xml)
        self.assertIn('无效的兑换码,或已被兑换过。', rsp)

        exc = Exchange(coupon=coupon, user=user, cost=10, status='WAITING',
                       exchange_code='1233')
        exc.save()

        xml = """<xml>
        <ToUserName><![CDATA[techparty]]></ToUserName>
        <FromUserName><![CDATA[o_BfQjrOWghP2cM0cN7K0kkR54fA]]></FromUserName>
        <CreateTime>1400131860</CreateTime>
        <MsgType><![CDATA[text]]></MsgType>
        <Content><![CDATA[bb1233]]></Content>
        <MsgId>1234567890123456</MsgId>
        </xml>"""
        rsp = self.request_content(xml)
        self.assertIn('您已获得了10金币', rsp)

        xml = """<xml>
        <ToUserName><![CDATA[techparty]]></ToUserName>
        <FromUserName><![CDATA[o_BfQjrOWghP2cM0cN7K0kkR54fA]]></FromUserName>
        <CreateTime>1400131860</CreateTime>
        <MsgType><![CDATA[text]]></MsgType>
        <Content><![CDATA[bb1233]]></Content>
        <MsgId>1234567890123456</MsgId>
        </xml>"""
        rsp = self.request_content(xml)
        self.assertIn('无效的兑换码,或已被兑换过。', rsp)


@skipIfCustomUser
class WechatpayTest(TestCase):

    def setUp(self):
        self.user = User.objects.create_user('jeff',
                                             'jeff@toraysoft.com', '123')
        self.plan = ChargePlan(name=u'plan1', code='plan1',
                               value=30, cost=750, coupon=50)
        self.plan.save()

    def test_gen_package(self):
        cli = Client()
        package = {'bank_type': 'WX', 'body': '千足 金箍棒',
                   'fee_type': '1', 'input_charset': 'UTF-8',
                   'getOutTradNo': '81282373272',
                   'spbill_create_ip': '192.168.1.1', 'total_fee': '1'}
        data = {'traceid': 'ikaoiaos', 'package': package}
        data = json.dumps(data)
        print 'data = %s' % data
        rsp = cli.post('/gc/wechatpaypackage/',
                       data=data, content_type='application/json')
        print rsp.content

    def test_xml_to_dict(self):
        from .views import _wechatpay_xml_to_dict
        raw_str = ("""
            <xml>
                <OpenId><![CDATA[111222]]></OpenId>
                <AppId><![CDATA[wwwwb4f85f3a797777]]></AppId>
                <IsSubscribe>1</IsSubscribe>
                <TimeStamp> 1369743511</TimeStamp>
                <NonceStr><![CDATA[jALldRTHAFd5Tgs5]]></NonceStr>
                <AppSignature><![CDATA[bafe07f060fdb4b5ff756f973aecffa]]>
                </AppSignature>
                <SignMethod><![CDATA[sha1]]></SignMethod >
            </xml>""")
        dict_ret = _wechatpay_xml_to_dict(raw_str)
        print dict_ret

    def test_wechatpay_get_info(self):
        plan = ChargePlan()
        plan.name = u'商品商品'
        plan.value = 10000
        plan.cost = 100
        plan.save()

        from goldencage.views import wechat_pay_get_access_token
        from goldencage.views import wechatpay_get_info
        content = wechat_pay_get_access_token()
        access_token = content.get('access_token')
        if not access_token:
            print content
        data, err = wechatpay_get_info(
            access_token, plan.id, '123321', '127.0.0.1', 'traceiddd')

    def test_wechatpay_notify(self):
        order = Order(id=1115, user=self.user, plan=self.plan, value=30)
        order.save()

        body = """
        <xml><OpenId><![CDATA[oaCDJju5TzPSv0ZT_GP5nLsPAQfY]]></OpenId>
<AppId><![CDATA[wx6745aaa6e2878f99]]></AppId>
<IsSubscribe>0</IsSubscribe>
<TimeStamp>1418365289</TimeStamp>
<NonceStr><![CDATA[kLI9t3MWRx4RYZVu]]></NonceStr>
<AppSignature><![CDATA[16d03d4341d62a3d635c7593cc84eb1554c36205]]></AppSignature>
<SignMethod><![CDATA[sha1]]></SignMethod>
</xml>
        """

        params = 'bank_billno=201412126100895338&bank_type=3006&discount=0&fee_type=1&input_charset=UTF-8&notify_id=epFRTtDSAK6AGztmmEb5cOpCQCzg06fiAj8D9w6l_0VbjHy2_6NnDpKIs5un-g5TJTsCCDC1ZA8jFy3WY2VV1nWNYehhK-Tg&out_trade_no=1115&partner=1222813501&product_fee=1&sign=6265C0C62683BE1F5F7C6D688A25CD00&sign_type=MD5&time_end=20141212142129&total_fee=1&trade_mode=1&trade_state=0&transaction_id=1222813501201412126039873136&transport_fee=0'
        url = '/gc/wechatcb/?%s' % params
        cli = Client()
        rsp = cli.post(url, data=body, content_type='application/xml')
        print '+++++++++++++++++'
        print rsp.content

    def test_wechatpay_mp_get_info(self):
        print '+++++++++++++++++'
        from goldencage.views import wechatpay_mp_get_info
        prepay_id, errmsg = wechatpay_mp_get_info(
            self.plan.id,
            out_trade_no='123',
            client_ip='127.0.0.1',
            openid='oFTfqjmMVWKo7GM0vuFhpJHWDjh4',
            trade_type='JSAPI'
        )
        print prepay_id
        print errmsg

    def test_wechat_mp_pay_verify(self):
        from goldencage.views import wechat_mp_pay_verify
        req_dict = {'openid': 'oFTfqjiGdMbL-6I04rqcU_PNziyg', 'trade_type': 'JSAPI', 'total_fee': '1', 'return_code': 'SUCCESS', 'nonce_str': 'lu5lr4a272iy5', 'is_subscribe': 'Y', 'fee_type': 'CNY', 'bank_type': 'CMB_CREDIT', 'mch_id':
                    '1229194702', 'out_trade_no': '277250', 'transaction_id': '1003760227201502030012096138', 'time_end': '20150203223430', 'appid': 'wx02dce087b6279278', 'sign': 'FF3BBBE7E99D6043510F85FCFC322B08', 'cash_fee': '1', 'result_code': 'SUCCESS'}
        wechat_mp_pay_verify(req_dict)

        body = """
        <xml><appid><![CDATA[wx02dce087b7279278]]></appid>
<bank_type><![CDATA[CMB_CREDIT]]></bank_type>
<cash_fee><![CDATA[1]]></cash_fee>
<fee_type><![CDATA[CNY]]></fee_type>
<is_subscribe><![CDATA[Y]]></is_subscribe>
<mch_id><![CDATA[12213494702]]></mch_id>
<nonce_str><![CDATA[lu5lr4a272iy5]]></nonce_str>
<openid><![CDATA[oFTfqjiGdMbL-6I04rqcU_PNziyg]]></openid>
<out_trade_no><![CDATA[277250]]></out_trade_no>
<result_code><![CDATA[SUCCESS]]></result_code>
<return_code><![CDATA[SUCCESS]]></return_code>
<sign><![CDATA[FF3BBBE7E99D6043510F85FCFC322B08]]></sign>
<time_end><![CDATA[20150203223430]]></time_end>
<total_fee>1</total_fee>
<trade_type><![CDATA[JSAPI]]></trade_type>
<transaction_id><![CDATA[1003760227201502030012096138]]></transaction_id>
</xml>
        """
        url = '/gc/wechatmpcb/'
        cli = Client()
        rsp = cli.post(url, data=body, content_type='application/xml')
        print '+++++++++++++++++'
        print rsp.content



# encoding=utf-8
from django.db import models
from django.db import IntegrityError
from django.conf import settings
from django.dispatch import Signal

from jsonfield import JSONField
import calendar
import time
from datetime import datetime
from goldencage import config
import random
import pytz
from hashlib import sha1

import logging
log = logging.getLogger(__name__)


task_done = Signal(providing_args=['cost', 'user'])
appwalllog_done = Signal(providing_args=['cost', 'user'])
payment_done = Signal(providing_args=['cost', 'user', 'plan', 'order'])
apply_coupon = Signal(providing_args=['instance', 'cost', 'user'])


class Task(models.Model):
    name = models.CharField(u'任务名称', max_length=50)
    key = models.CharField(u'代码', max_length=50, unique=True)
    cost = models.IntegerField(u'金币', default=0)
    cost_max = models.IntegerField(
        u'最大金币', default=0,
        help_text=u'如不为0，实际所得为"金币"与"最大金币"之间的随机值')
    interval = models.IntegerField(
        u'时间间隔',
        default=0,
        help_text=u'两次任务之间的时间间隔（秒），0为不限')
    limit = models.IntegerField(
        u'次数上限',
        default=0,
        help_text=u'任务允许执行的最大次数，0为不限')
    daily = models.BooleanField(u'允许每天一次', default=False)

    def _save_log(self, user, valid=True, cost=None):
        if not cost:
            if self.cost_max > 0 and self.cost < self.cost_max:
                cost = random.randint(self.cost, self.cost_max)
        cost = cost or self.cost

        log = TaskLog(user=user, job=self, valid=valid,
                      cost=cost if valid else 0)
        log.save()
        if valid:
            task_done.send(sender=Task, cost=log.cost, user=user)
        return log

    def make_log(self, user, cost=None):
        last = TaskLog.objects.filter(
            user=user, job=self, valid=True).order_by('-create_time')

        if not last:
            return self._save_log(user, cost=cost)

        if self.limit > 0:
            if last.count() >= self.limit:
                return self._save_log(user, False, cost=cost)

        if self.interval > 0:
            last_time = calendar.timegm(last[0].create_time.timetuple())
            if (time.time() - last_time) <= self.interval:
                return self._save_log(user, False, cost=cost)
        if self.daily:
            d1 = datetime.now()
            d2 = last[0].create_time.replace(tzinfo=pytz.utc)\
                .astimezone(pytz.timezone(settings.TIME_ZONE))
            log.info('date 1 %s, date 2 %s' % (d1, d2))
            if d1.date() <= d2.date():
                return self._save_log(user, False, cost=cost)

        return self._save_log(user, cost=cost)

    def __unicode__(self):
        return self.name

    class Meta:
        verbose_name = u'任务类型'
        verbose_name_plural = u'任务类型'


class TaskLog(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL)
    job = models.ForeignKey(Task)
    cost = models.IntegerField()
    create_time = models.DateTimeField(auto_now_add=True)
    valid = models.BooleanField(default=True)

    class Meta:
        verbose_name = u'任务纪录'
        verbose_name_plural = u'任务纪录'


class AppWallLog(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL)
    provider = models.CharField(max_length=20,
                                choices=(('youmi_ios', u'有米iOS'),
                                         ('youmi_adr', u'有米Android'),
                                         ('waps', u'万普'),
                                         ('dianjoy_adr', u'点乐'),
                                         ('qumi', u'趣米iOS'),
                                         ('qumi_adr', u'趣米Android'),
                                         ('domob_adr', u'多盟Android'),
                                         ('domob_ios', u'多盟iOS'),
                                         ))
    identity = models.CharField(max_length=100)
    cost = models.IntegerField()
    product_id = models.CharField(max_length=100)
    product_name = models.CharField(max_length=100)
    create_time = models.DateTimeField(auto_now_add=True)
    valid = models.BooleanField(default=True)
    extra_data = JSONField(blank=True, null=True)

    class Meta:
        verbose_name = u'积分墙纪录'
        verbose_name_plural = u'积分墙纪录'

        unique_together = (('user', 'provider', 'identity'),)

    @classmethod
    def log(cls, data, provider):
        if provider not in config.APPWALLLOG_MAPPING:
            raise ValueError('unknown appwall provider')
        mapping = config.APPWALLLOG_MAPPING[provider]
        alog = AppWallLog(provider=provider)
        for key, value in mapping.iteritems():
            if isinstance(value, tuple):
                value = '_'.join([data.get(v, '') for v in value])
            else:
                value = data[value]
            if key in ('user_id', 'cost'):
                try:
                    value = int(value)
                except:
                    return True
            setattr(alog, key, value)
        alog.extra_data = data
        # 超出的唯一标识，将换成为sha1字符串再存储。
        if len(alog.identity) >= 100:
            alog.identity = sha1(alog.identity).hexdigest()
        try:
            alog.save()
            appwalllog_done.send(sender=cls, cost=alog.cost, user=alog.user)
            return alog
        except IntegrityError:
            return None


class ChargePlan(models.Model):
    name = models.CharField(u'标题', max_length=50)
    value = models.IntegerField(u'价值')
    cost = models.IntegerField(u'对应积分')
    coupon = models.IntegerField(u'赠送积分', default=0)
    valid = models.BooleanField(u'有效', default=True)
    code = models.CharField(u'商品代码', max_length=50)

    def __unicode__(self):
        return self.name

    class Meta:
        verbose_name = u'充值套餐'
        verbose_name_plural = u'充值套餐'


class Order(models.Model):
    plan = models.ForeignKey(ChargePlan, verbose_name='套餐')
    user = models.ForeignKey(settings.AUTH_USER_MODEL, verbose_name=u'用户')
    platform = models.CharField(u'支付平台', max_length=20)
    create_time = models.DateTimeField(auto_now_add=True)
    value = models.IntegerField(u'金额(单位：分)')
    status = models.IntegerField(default=0,
                                 choices=((0, u'已下订'), (1, u'已支付'),
                                          (2, u'过期未支付')))

    def __unicode__(self):
        return self.plan.name

    @classmethod
    def get_real_id(cls, oid):
        test_id = str(oid)
        if len(test_id) < 9:
            return oid

        prefix = getattr(settings, 'PERAKCAGE_ORDER_ID_PREFIX', 0)
        if not prefix:
            return oid
        prefix = str(prefix)
        if not test_id.startswith(prefix):
            return oid

        return int(test_id[len(prefix):])

    def gen_order_id(self):
        prefix = getattr(settings, 'PERAKCAGE_ORDER_ID_PREFIX', 0)
        if not prefix:
            return self.id
        try:
            prefix = int(prefix)
        except:
            return self.id

        prefix = str(prefix)
        rid = str(self.id)
        paddings = '0' * (9 - len(prefix) - len(rid))
        return int(prefix + paddings + rid)

    class Meta:
        verbose_name = u'订单'
        verbose_name_plural = u'订单'


class Charge(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL, verbose_name=u'用户')
    platform = models.CharField(u'充值平台', max_length=20)
    account = models.CharField(u'充值帐号', max_length=50)
    email = models.CharField(u'充值帐号email', max_length=50,
                             blank=True, null=True, db_index=True)
    value = models.IntegerField(u'充入金额(单位：分)')
    cost = models.IntegerField(u'价值积分')
    transaction_id = models.CharField(u'平台交易号', max_length=100)
    order_id = models.CharField(u'交易号', max_length=100, unique=True)
    create_time = models.DateTimeField(u'交易时间', auto_now_add=True)
    valid = models.BooleanField(u'是否有效', default=True)
    status = models.CharField(u'状态', max_length=50, blank=True, null=True)
    extra_data = JSONField(u'交易源数据', blank=True, null=True)

    class Meta:
        verbose_name = u'充值纪录'
        verbose_name_plural = u'充值纪录'

        unique_together = (('platform', 'transaction_id'),)

    def is_finish(self):
        return self.status == config.PAYMENT_FINISH[self.platform]

    def value_in_cent(self, value):
        return int(value * config.PAYMENT_SCALE[self.platform])

    @classmethod
    def recharge(cls, data, provider):

        def dispatch_signal(cost, user, plan, order):
            payment_done.send(sender=cls, cost=cost,
                              user=user, plan=plan, order=order)
            if plan and plan.coupon > 0:
                try:
                    task = Task.objects.get(key='__recharge')
                except Task.DoesNotExist:
                    task = Task(name=u'充值', key='__recharge')
                    task.save()
                task.make_log(user, cost=plan.coupon)
            order.status = 1
            order.save()

        if provider not in config.PAYMENT_MAPPING:
            raise ValueError('no mapping for %s' % provider)
        mapping = config.PAYMENT_MAPPING[provider]
        chg = Charge(platform=provider)
        for key, value in mapping.iteritems():
            value = data[value]
            if key == 'value':
                value = chg.value_in_cent(float(value))
            setattr(chg, key, value)
        chg.extra_data = data

        order = Order.objects.get(pk=Order.get_real_id(chg.order_id))

        if order.value != chg.value:
            log.info('Order.value = %s' % order.value)
            log.info('chg.value = %s' % chg.value)
            log.error(u'充值的金额与套餐不匹配，无效 %s' % chg.order_id)
            return None
        plan = order.plan
        chg.cost = plan.cost
        chg.user = order.user
        chg.valid = False

        result = Charge.objects.filter(platform=chg.platform,
                                       transaction_id=chg.transaction_id)
        if result:
            result = result[0]
            if result.status == chg.status:
                return None
            else:
                if result.is_finish():
                    # 已经完毕，这条旧通知是来晚了。忽略掉
                    return result

                # 还没完结，继续修改状态。
                if chg.is_finish():
                    result.valid = True
                    dispatch_signal(result.cost, result.user, plan, order)
                # 更新状态
                result.status = chg.status
                result.save()
                return result
        else:
            try:
                if chg.is_finish():
                    chg.valid = True
                    chg.save()
                    dispatch_signal(chg.cost, chg.user, plan, order)
                else:
                    # 怕signal的connect方出了错, 写两个save吧
                    chg.save()

                return chg
            except IntegrityError:
                return None


class Coupon(models.Model):

    """优惠券，用于各种活动兑换金币用。
    """
    name = models.CharField(u'名称', max_length=50)
    key = models.CharField(u'代码', max_length=50)
    cost = models.IntegerField(u'价值积分', default=0)
    disable = models.BooleanField(u'禁用', default=False)
    create_time = models.DateTimeField(auto_now_add=True)

    exchange_style = models.CharField(u'兑换方式', default=u'wechat',
                                      max_length=10)
    exchange_config = JSONField(blank=True, null=True)

    # 限制
    limit = models.IntegerField(u'每个限制次数', default=1,
                                help_text=u'值为0时不限')

    def generate(self, user, default=None):
        """为用户生成一张优惠券,
        """

        # 看看该优惠券是否限制次数，有限制，同时
        if self.limit:
            exchanges = Exchange.objects.filter(
                coupon=self, user=user, status='DONE').count()
            if exchanges >= self.limit:
                return None

        waitings = Exchange.objects.filter(
            coupon=self, user=user, status='WAITING')
        if waitings:
            return waitings[0]
        else:
            max_value = getattr(settings, 'PERAKCAGE_COUPONCODE_MAX',
                                999999)
            code = default or random.randint(1000, max_value)
            while True:
                test = Exchange.objects.filter(exchange_code=str(code),
                                               coupon=self, status='WAITING')
                if not test:
                    break
                code = random.randint(1000, max_value)
            exchange = Exchange(coupon=self, user=user, cost=self.cost,
                                exchange_code=str(code))
            exchange.save()
            return exchange

    def validate(self, code, user=None):
        """ 验证优惠券
        """
        exchange = Exchange.objects.filter(
            coupon=self, exchange_code=code, status='WAITING')
        if not exchange:
            return False
        exchange = exchange[0]
        exchange.status = 'DONE'
        exchange.exchange_user = user
        exchange.cost = self.cost
        exchange.save()
        apply_coupon.send(sender=Coupon, instance=self,
                          cost=exchange.cost, user=exchange.user)
        return exchange

    class Meta:
        verbose_name = u'礼券'
        verbose_name_plural = u'礼券'

    def __unicode__(self):
        return self.name


class Exchange(models.Model):

    """优惠券兑换纪录
    """
    coupon = models.ForeignKey(Coupon, verbose_name=u'优惠券')
    user = models.ForeignKey(settings.AUTH_USER_MODEL,
                             verbose_name=u'用户')
    exchange_code = models.CharField(u'兑换码', max_length=50,
                                     blank=True, null=True,
                                     db_index=True)
    status = models.CharField(u'状态', max_length=10,
                              default='WAITING',
                              choices=(('WAITING', u'等待兑换'),
                                       ('DONE', u'兑换完成')))
    cost = models.IntegerField(u'获得积分', default=0)
    create_time = models.DateTimeField(auto_now_add=True)
    update_time = models.DateTimeField(auto_now=True)
    exchange_user = models.CharField(u'兑换用户', max_length=200,
                                     blank=True, null=True)

    class Meta:
        verbose_name = u'礼券兑换纪录'
        verbose_name_plural = u'礼券兑换纪录'

    def __unicode__(self):
        return self.coupon.name



# encoding=utf-8

from django.contrib import admin

from perakcagecage.models import Task
from perakcagecage.models import ChargePlan
from perakcagecage.models import TaskLog
from perakcagecage.models import Charge
from perakcagecage.models import Order
from perakcageage.models import AppWallLog
from perakcagecage.models import Coupon
from perakcagecage.models import Exchange


class TaskLogAdmin(admin.ModelAdmin):
    list_display = ['job', 'cost', 'create_time', 'valid']
    list_filter = ('job', 'create_time')
    raw_id_fields = ('user',)
    search_fields = ('user__id',)
    list_select_related = ['job']


class AppWallLogAdmin(admin.ModelAdmin):
    list_display = ['provider', 'cost', 'product_name', 'create_time', 'valid']
    list_filter = ('provider', 'create_time')
    raw_id_fields = ('user',)
    search_fields = ('user__id',)
    list_select_related = []


class OrderAdmin(admin.ModelAdmin):
    list_display = ('plan', 'value', 'status', 'create_time')
    list_filter = ('plan', 'create_time', 'status')
    raw_id_fields = ('user',)
    search_fields = ('user__id',)
    list_select_related = ['plan']


class ChargeAdmin(admin.ModelAdmin):
    list_display = ('account', 'email', 'value', 'cost',
                    'create_time', 'status')
    list_filter = ('create_time', 'status')
    raw_id_fields = ('user',)
    search_fields = ('email', 'user__id')
    list_select_related = []


class CouponAdmin(admin.ModelAdmin):
    list_display = ('name', 'key', 'cost', 'exchange_style',
                    'limit', 'disable')
    list_filter = ('exchange_style', 'disable')


class ExchangeAdmin(admin.ModelAdmin):
    list_display = ('coupon', 'exchange_code', 'status', 'cost', 'create_time')
    list_filter = ('status', 'create_time')
    raw_id_fields = ('user',)
    search_fields = ('user__id', 'exchange_code')
    list_select_related = []


class ChargePlanAdmin(admin.ModelAdmin):
    list_display = ('name', 'value', 'cost', 'coupon', 'valid', 'code')


admin.site.register(Task, admin.ModelAdmin)
admin.site.register(ChargePlan, ChargePlanAdmin)
admin.site.register(TaskLog, TaskLogAdmin)
admin.site.register(Charge, ChargeAdmin)
admin.site.register(Order, OrderAdmin)
admin.site.register(AppWallLog, AppWallLogAdmin)
admin.site.register(Coupon, CouponAdmin)
admin.site.register(Exchange, ExchangeAdmin)



# encoding=utf-8

from django.http import HttpResponseForbidden
from django.http import HttpResponse
from django.http import HttpResponseNotAllowed
from django.conf import settings
from django.views.decorators.csrf import csrf_exempt
from django.core.cache import cache

from Crypto.PublicKey import RSA
from Crypto.Signature import PKCS1_v1_5
from Crypto.Hash import SHA

import base64
import hashlib
import urllib
import requests
import simplejson as json
from random import Random
import time
from xml.dom import minidom
from dicttoxml import dicttoxml
import xmltodict

from goldencage.models import AppWallLog
from goldencage.models import Charge
from goldencage import config
from goldencage.models import Coupon
from goldencage.models import ChargePlan


from wechat.official import WxApplication, WxTextResponse, WxResponse

import logging
log = logging.getLogger('django')


def rsp(data=None):
    if data is None:
        data = {}
    ret = {'errcode': 0, 'errmsg': 'ok', 'data': data}
    ret = json.dumps(ret)
    return HttpResponse(ret, content_type='application/json')


def error_rsp(code, msg, data=None):
    if data is None:
        data = {}
    ret = {'errcode': code, 'errmsg': msg, 'data': data}
    ret = json.dumps(ret)
    return HttpResponse(
        {'errcode': code, 'errmsg': msg, 'data': data},
        content_type='application/json'
    )


waps_ips = ['219.234.85.238', '219.234.85.223',
            '219.234.85.211', '219.234.85.231',
            '127.0.0.1']


def waps_callback(request):
    ip = request.META.get("REMOTE_ADDR", None)

    if ip not in waps_ips and not request.GET.get('debug', None):
        return HttpResponseNotAllowed("incorrect IP address")
    wapslog = {}
    for key in request.GET.keys():
        wapslog[key] = request.GET[key]
    if AppWallLog.log(wapslog, provider='waps'):
        return HttpResponse(json.dumps(
            {"message": u"成功接收", "success": True}))
    else:
        return HttpResponse(json.dumps(
            {"message": u"无效数据", "success": False}))


def youmi_callback_adr(request):
    sign = request.GET.get('sig')
    if not sign:
        return HttpResponseForbidden("miss param 'sign'")

    keys = ['order', 'app', 'user', 'chn', 'ad', 'points']
    vals = [request.GET.get(k, '').encode('utf8').decode('utf8') for k in keys]
    vals.insert(0, settings.YOUMI_CALLBACK_SECRET_ADR)
    token = u'||'.join(vals)
    md5 = hashlib.md5()
    md5.update(token.encode('utf-8'))
    md5 = md5.hexdigest()
    _sign = md5[12:20]

    if sign != _sign:
        return HttpResponseForbidden("signature error")
    youmilog = {}
    for key in keys:
        youmilog[key] = request.GET[key]
    if AppWallLog.log(youmilog, provider='youmi_adr'):
        return HttpResponse('OK')
    else:
        return HttpResponseForbidden('already exist')

    return HttpResponseForbidden("Signature verification fail")


def youmi_callback_ios(request):
    sign = request.GET.get('sign')
    if not sign:
        return HttpResponseForbidden("miss param 'sign'")

    keys = request.GET.keys()
    keys.sort()

    src = ''.join(['%s=%s' %
                   (k, request.GET.get(k).encode('utf-8').decode('utf-8'))
                   for k in keys if k != 'sign'])
    src += settings.YOUMI_CALLBACK_SECRET
    md5 = hashlib.md5()
    md5.update(src.encode('utf-8'))
    _sign = md5.hexdigest()

    if sign != _sign:
        return HttpResponseForbidden("signature error")

    youmilog = {}
    for key in keys:
        youmilog[key] = request.GET[key]
    if AppWallLog.log(youmilog, provider='youmi_ios'):
        return HttpResponse('OK')
    else:
        return HttpResponseForbidden('already exist')

    return HttpResponseForbidden("Signature verification fail")


def dianjoy_callback_adr(request):
    token = request.GET.get('token')
    time_stamp = request.GET.get('time_stamp')
    md5 = hashlib.md5()
    md5.update(time_stamp + settings.GOLDENCAGE_DIANJOY_ANDROID_SECRET)
    sign = md5.hexdigest()
    if sign != token:
        return HttpResponseForbidden('token error')
    log = {}
    for key in request.GET.keys():
        log[key] = request.GET[key]
    if AppWallLog.log(log, provider='dianjoy_adr'):
        return HttpResponse('200')
    else:
        return HttpResponse('OK, But duplicate item')


def qumi_callback(request):
    sig = request.GET.get('sig')
    log = {}
    for key in request.GET.keys():
        log[key] = request.GET[key]
    src = '||'.join([log[key] for key in
                     ('order', 'app', 'ad', 'user', 'device',
                      'points', 'time')]).encode('utf-8')
    md5 = hashlib.md5()
    md5.update(settings.GOLDENCAGE_QUMI_SECRET + '||' + src)
    _sig = md5.hexdigest()[8:24]
    if sig != _sig:
        return HttpResponseForbidden('sig error')
    if AppWallLog.log(log, provider='qumi'):
        return HttpResponse('OK')
    else:
        return HttpResponse('OK, But Duplicated item')


def qumi_callback_android(request):
    sig = request.GET.get('sig')
    log = {}
    for key in request.GET.keys():
        log[key] = request.GET[key]
    src = '||'.join([log[key] for key in
                     ('order', 'app', 'ad', 'user', 'device',
                      'points', 'time')]).encode('utf-8')
    md5 = hashlib.md5()
    md5.update(settings.GOLDENCAGE_QUMI_SECRET_ANDROID + '||' + src)
    _sig = md5.hexdigest()[8:24]
    if sig != _sig:
        return HttpResponseForbidden('sig error')

    if AppWallLog.log(log, provider='qumi_adr'):
        return HttpResponse('OK')
    else:
        return HttpResponse('OK, But Duplicated item')


def domob_callback_sign(request, private_key):
    params = {}
    for k, v in request.GET.iteritems():
        if k != 'sign':
            params[k] = v
    param_list = sorted(params.iteritems(), key=lambda d: d[0])
    sign = ''
    for param in param_list:
        sign += (str(param[0]) + '=' + str(param[1]))
    sign += str(private_key)
    m = hashlib.md5()
    m.update(sign)
    return m.hexdigest()


def domob_callback_hdl(request, private_key, provider):
    log.info(
        'request.GET = {GET} provider = {provider}'.format(
            GET=request.GET, provider=provider)
    )
    signStr = domob_callback_sign(request, private_key)
    sign = request.GET.get('sign', '')
    if sign != signStr:
        return HttpResponseForbidden()
    else:
        if AppWallLog.log(request.GET, provider=provider):
            return HttpResponse('OK')
        else:
            return HttpResponse('OK, But Duplicated item')


def domob_callback_android(request):
    return domob_callback_hdl(
        request,
        settings.GOLDENCAGE_DOMOB_PRIVATE_KEY_ANDROID,
        'domob_adr',
    )


def domob_callback_ios(request):
    return domob_callback_hdl(
        request,
        settings.GOLDENCAGE_DOMOB_PRIVATE_KEY_IOS,
        'domob_ios',
    )


def appwall_callback(request, provider):
    return {'waps': waps_callback,
            'youmi_ios': youmi_callback_ios,
            'youmi_adr': youmi_callback_adr,
            'dianjoy_adr': dianjoy_callback_adr,
            'qumi': qumi_callback,
            'qumi_adr': qumi_callback_android,
            'domob_adr': domob_callback_android,
            'domob_ios': domob_callback_ios,
            }[provider](request)

alipay_public_key = config.ALIPAY_PUBLIC_KEY


# 支付宝回调 ########

def verify_notify_id(notify_id):
    # 检查是否合法的notify_id, 检测该id是否已被成功处理过。

    url = 'https://mapi.alipay.com/gateway.do'
    params = {'service': 'notify_verify',
              'partner': settings.ALIPAY_PID,
              'notify_id': notify_id}
    log.info('start verify notify_id %s' % notify_id)
    try:
        rsp = requests.get(url, params=params, timeout=5)
    except:
        log.error('timeout verify notify_id %s' % notify_id)
        return False
    log.info('finish verify notifi_id %s' % notify_id)
    return rsp.status_code == 200 and rsp.text == 'true'


def verify_alipay_signature(sign_type, sign, params):
    if sign_type == 'RSA':
        return rsa_verify(params, sign)
    else:
        return True


def filter_para(paras):
    """过滤空值和签名"""
    for k, v in paras.items():
        if not v or k in ['sign', 'sign_type']:
            paras.pop(k)
    return paras


def create_link_string(paras, sort, encode):
    """对参数排序并拼接成query string的形式"""
    if sort:
        paras = sorted(paras.items(), key=lambda d: d[0])
    if encode:
        return urllib.urlencode(paras)
    else:
        if not isinstance(paras, list):
            paras = list(paras.items())
        ps = ''
        for p in paras:
            if ps:
                ps = '%s&%s=%s' % (ps, p[0], p[1])
            else:
                ps = '%s=%s' % (p[0], p[1])
        return ps


def rsa_verify(paras, sign):
    """对签名做rsa验证"""
    log.debug('init paras = %s' % paras)
    pub_key = RSA.importKey(config.ALIPAY_PUBLIC_KEY)
    paras = filter_para(paras)
    paras = create_link_string(paras, True, False)
    log.debug('type(paras) = %s paras = %s' % (type(paras), paras))
    verifier = PKCS1_v1_5.new(pub_key)
    data = SHA.new(paras.encode('utf-8'))
    return verifier.verify(data, base64.b64decode(sign))


@csrf_exempt
def alipay_callback(request):
    # 支付宝支付回调，先检查签名是否正确，再检查是否来自支付宝的请求。
    # 有效的回调，将更新用户的资产。
    keys = request.REQUEST.keys()
    data = {}
    for key in keys:
        data[key] = request.REQUEST[key]
    notify_id = data['notify_id']
    sign_type = data['sign_type']
    sign = data['sign']
    order_id = data['out_trade_no']

    log.info(u'alipay callback, order_id: %s , data: %s' % (order_id, data))

    nid = cache.get('ali_nid_' + hashlib.sha1(notify_id).hexdigest())
    if nid:
        log.info('duplicated notify, drop it')
        return HttpResponse('error')

    if verify_notify_id(notify_id) \
            and verify_alipay_signature(sign_type, sign, data) \
            and Charge.recharge(data, provider='alipay'):
        cache.set('ali_nid_' + hashlib.sha1(notify_id).hexdigest(),
                  order_id, 90000)  # notify_id 保存25小时。
        log.info('ali callback success')
        return HttpResponse('success')
    log.info('not a valid callback, ignore')
    return HttpResponse('error')


def rsa_sign(para_str):
    """对请求参数做rsa签名"""
    para_str = para_str.encode('utf-8')
    key = RSA.importKey(settings.ALIPAY_PRIVATE_KEY)
    h = SHA.new(para_str)
    signer = PKCS1_v1_5.new(key)
    return base64.b64encode(signer.sign(h))


@csrf_exempt
def alipay_sign(request):
    if request.method != 'POST':
        logging.error('equest.method != "POST"')
        return error_rsp(5099, 'error')

    log.info('request.POST = %s' % request.POST)
    log.info('request.body = %s' % request.body)

    try:
        content = json.loads(request.body)
    except:
        content = request.POST

    words = content.get('words')
    if not words:
        logging.error('if not words')
        return error_rsp(5099, 'error')

    sign_type = content.get('sign_type')
    if not sign_type:
        sign_type = 'RSA'

    if sign_type == 'RSA':
        en_str = rsa_sign(words)
    else:
        en_str = ''

    data = {'en_words': en_str}
    return rsp(data)


class WxEmptyResponse(WxResponse):

    def as_xml(self):
        return ''


class ChatView(WxApplication):
    SECRET_TOKEN = getattr(settings, 'GOLDENCAGE_WECHAT_TOKEN', '')
    BALANCE_UNIT_NAME = getattr(settings, 'GOLDENCAGE_BALANCE_UNIT_NAME',
                                u'金币')
    SUCCESS_MESSAGE_TEMPLATE = getattr(
        settings, 'GOLDENCAGE_COUPONE_SUCCESS_MESSAGE_TEMPLATE',
        u'您已获得了%d%s')

    def on_text(self, text):
        content = text.Content.lower()
        coupons = Coupon.objects.filter(disable=False, exchange_style='wechat')
        for cp in coupons:
            if content.startswith(cp.key):
                content = content.replace(cp.key, '').strip()
                result = cp.validate(content)
                if result:
                    return WxTextResponse(
                        self.SUCCESS_MESSAGE_TEMPLATE %
                        (cp.cost, self.BALANCE_UNIT_NAME), text)
                else:
                    return WxTextResponse(u'无效的兑换码,或已被兑换过。',
                                          text)
        return WxEmptyResponse(text)


@csrf_exempt
def wechat(request):
    """只处理文本，并且只处理一个命令。
    """
    app = ChatView()
    if request.method == 'GET':
        # 用于校验访问权限, 直接返回一字符串即可。
        rsp = app.process(request.GET)
        return HttpResponse(rsp)
    elif request.method == 'POST':
        rsp = app.process(request.GET, request.body)
        if not rsp:
            return HttpResponse('')
        return HttpResponse(rsp)


# 微信支付
WECHATPAY_ACCESS_TOKEN_URL = 'https://api.weixin.qq.com/cgi-bin/token'


def wechat_pay_get_access_token():
    params = {
        'grant_type': 'client_credential',
        'appid': settings.WECHATPAY_APPID,
        'secret': settings.WECHATPAY_SECRET
    }
    log.debug('params = %s' % params)
    rsp = requests.get(WECHATPAY_ACCESS_TOKEN_URL, params=params, verify=False)
    content = json.loads(rsp.content)
    errcode = content.get('errcode')
    if errcode:
        log.error(errcode)
        log.error(content['errmsg'])
        return {}
    else:
        return content


@csrf_exempt
def wechat_pay_gen_package(request):
    if request.method != 'POST':
        logging.error('equest.method != "POST"')
        return error_rsp(5099, 'error')

    log.debug(u'request.body = %s' % request.body)
    log.debug(u'body type = %s' % type(request.body))
    body = json.loads(request.body)
    package = body.get('package')
    if not package:
        return error_rsp(5099, 'error')
    package = _wechatpay_gen_package(package)
    log.debug(u'package = %s' % package)

    noncestr = random_str(13)
    timestamp = '%.f' % time.time()
    traceid = body.get('traceid', '')
    sha_param = {
        'appid': settings.WECHATPAY_APPID,
        'appkey': settings.WECHATPAY_APPKEY,
        'noncestr': noncestr,
        'package': package,
        'timestamp': timestamp,
        'traceid': traceid}
    app_signature = _wechatpay_app_signature(sha_param)
    log.debug(u'app_signature = %s' % app_signature)

    data = {'package': package}
    data['appid'] = settings.WECHATPAY_APPID
    data['noncestr'] = noncestr
    data['traceid'] = traceid
    data['timestamp'] = timestamp
    data['sign_method'] = 'sha1'
    data['app_signature'] = app_signature
    data['partnerid'] = settings.WECHATPAY_PARTNERID
    data['appkey'] = settings.WECHATPAY_APPKEY

    log.debug(u'rsp data = %s' % data)

    return rsp(data)


def convert_params_to_str_in_order(params, urlencode=False):
    param_list = sorted(params.iteritems(), key=lambda d: d[0])
    tmp_str = u''
    for val in param_list:
        vall = u'%s' % val[1]
        if urlencode:
            vall = vall.encode('utf-8')
            vall = urllib.quote(vall)
        if tmp_str:
            append_str = u'&%s=%s' % (val[0], vall)
            tmp_str = tmp_str + append_str
        else:
            tmp_str = u'%s=%s' % (val[0], vall)
    return tmp_str.encode('utf-8')


def _wechatpay_gen_package(
        package=None, body=None, out_trade_no=None,
        total_fee=None, ip=None):
    if not package:
        package = {}
    package['bank_type'] = 'WX'
    package['body'] = body or package.get('body') or ''
    # package['attach'] = ''
    package['partner'] = settings.WECHATPAY_PARTNERID
    package['out_trade_no'] = out_trade_no or package.get('out_trade_no') or ''
    package['total_fee'] = total_fee or package.get('total_fee') or ''
    package['fee_type'] = 1
    package['notify_url'] = settings.WECHATPAY_NOTIFY_URL
    package['spbill_create_ip'] = ip or package.get('spbill_create_ip') or ''
    # package['time_start'] = ''
    # package['time_expire'] = ''
    # package['transport_fee'] = ''
    # package['product_fee'] = ''
    # package['goods_tag'] = ''
    package['input_charset'] = 'UTF-8'

    string1 = convert_params_to_str_in_order(package)
    stringSignTemp = string1 + '&key=%s' % settings.WECHATPAY_PARTNERKEY
    log.debug(type(stringSignTemp))
    log.debug('stringSignTemp = %s' % stringSignTemp)
    md5 = hashlib.md5()
    md5.update(stringSignTemp)
    sign_str = md5.hexdigest().upper()
    log.debug(u'sign = %s' % sign_str)
    string1 = convert_params_to_str_in_order(package, urlencode=True)
    package = string1 + '&sign=%s' % sign_str
    return package


def _wechatpay_app_signature(params):
    params_str = convert_params_to_str_in_order(params)
    sign = hashlib.sha1(params_str).hexdigest()
    return sign


def random_str(randomlength=8):
    str_ = ''
    chars = 'abcdefghijklmnopqrstuvwxyz0123456789'
    length = len(chars) - 1
    random = Random()
    for i in range(randomlength):
        str_ += chars[random.randint(0, length)]
    return str_


def wechatpay_prepayid_params(planid, out_trade_no, client_ip, traceid):
    plan = ChargePlan.objects.get(pk=int(planid))
    package = _wechatpay_gen_package(
        package=None, body=plan.name, out_trade_no=out_trade_no,
        total_fee=plan.value, ip=client_ip)
    # package = _wechatpay_gen_package(
    #     package=None, body=plan.name, out_trade_no=out_trade_no,
    #     total_fee=1, ip=client_ip)
    noncestr = random_str(13)
    timestamp = '%.f' % time.time()
    sha_param = {
        'appid': settings.WECHATPAY_APPID,
        'appkey': settings.WECHATPAY_APPKEY,
        'noncestr': noncestr,
        'package': package,
        'timestamp': timestamp,
        'traceid': traceid}
    app_signature = _wechatpay_app_signature(sha_param)
    log.debug(u'app_signature = %s' % app_signature)

    data = {'package': package}
    data['appid'] = settings.WECHATPAY_APPID
    data['noncestr'] = noncestr
    data['traceid'] = traceid
    data['timestamp'] = timestamp
    data['sign_method'] = 'sha1'
    data['app_signature'] = app_signature

    log.debug(u'rsp data = %s' % data)

    return data


def wechatpay_sign_result(noncestr, prepayid, timestamp):
    """ 要不要去掉 最后一个 &
    """
    raw_str = (
        u'appid=%s&appkey=%s&noncestr=%s&package=Sign=WXPay&'
        u'partnerid=%s&prepayid=%s&timestamp=%s'
        % (
            settings.WECHATPAY_APPID,
            settings.WECHATPAY_APPKEY,
            noncestr,
            settings.WECHATPAY_PARTNERID,
            prepayid,
            str(timestamp)
        )
    )
    log.debug('raw_str = %s' % raw_str)
    signResult = hashlib.sha1(raw_str).hexdigest()
    return signResult


def wechatpay_get_info(
        access_token, planid,
        out_trade_no, client_ip, traceid):
    """
    提供外部调用的接口
    traceid 商家对用户的唯一标识,如果用微信 SSO,此处建议填写 授权用户的 openid
    """

    data = wechatpay_prepayid_params(
        planid, out_trade_no, client_ip, traceid)

    params = {
        'access_token': access_token
    }
    url = 'https://api.weixin.qq.com/pay/genprepay'
    headers = {
        'content-type': 'application/json'
    }
    post_data = json.dumps(data)
    rsp = requests.post(url, params=params, data=post_data,
                        headers=headers, verify=False)
    content = json.loads(rsp.content)
    if content['errcode'] != 0:
        log.error(content['errmsg'])
        return None, content['errcode']
    signResult = wechatpay_sign_result(
        data['noncestr'],
        content['prepayid'],
        data['timestamp'])

    wechatpay_data = {}
    wechatpay_data['partnerid'] = settings.WECHATPAY_PARTNERID
    wechatpay_data['prepayid'] = content['prepayid']
    wechatpay_data['appid'] = settings.WECHATPAY_APPID
    wechatpay_data['package'] = 'Sign=WXPay'
    wechatpay_data['noncestr'] = data['noncestr']
    wechatpay_data['timestamp'] = data['timestamp']
    wechatpay_data['sign'] = signResult

    log.info('wechatpay_data = %s' % wechatpay_data)
    return wechatpay_data, None


@csrf_exempt
def wechat_pay_notify(request):
    if request.method != 'POST':
        logging.error('equest.method != "POST"')
        return HttpResponse('fail')
    if not _wechatpay_verify_notify(request.GET):
        return HttpResponse('fail')

    log.debug(u'type trade_state = %s' % type(request.GET['trade_state']))

    notify_id = request.GET['notify_id']
    order_id = request.GET['out_trade_no']
    log.info(u'request.GET = %s' % request.GET)
    log.info(u'wechatpay callback, order_id: %s' % order_id)
    log.info(u'request.body = %s' % request.body)

    nid = cache.get('wechatpay_nid_' + hashlib.sha1(notify_id).hexdigest())
    if nid:
        log.info(u'duplicated notify, drop it')
        return HttpResponse('fail')
    body_dict = _wechatpay_xml_to_dict(request.body)
    data = {}
    for key, item in request.GET.items():
        data[key] = item
    for key, item in body_dict.iteritems():
        data[key] = item
    data['trade_state'] = str(data['trade_state'])
    data['total_fee'] = data['total_fee']
    log.debug(u'Charge.recharge data = %s' % data)
    if Charge.recharge(data, provider='wechatpay'):
        cache.set('wechatpay_nid_' + hashlib.sha1(notify_id).hexdigest(),
                  order_id, 90000)  # notify_id 保存25小时。
        log.info(u'wechatpay callback success')
        return HttpResponse('success')

    log.info(u'not a valid callback, ignore')
    return HttpResponse('fail')


def _wechatpay_xml_to_dict(content):
    """ 只能解析一层xml，如果多层的话，最好改成用库 xmltodict
    """
    xml_dict = {}
    doc = minidom.parseString(content)
    params = [ele for ele in doc.childNodes[0].childNodes
              if isinstance(ele, minidom.Element)]
    for param in params:
        if param.childNodes:
            text = param.childNodes[0]
            xml_dict[param.tagName] = text.data
        else:
            xml_dict[param.tagName] = ''

    return xml_dict


def para_filter(params):
    return dict([(key, params[key])
                 for key in params
                 if key.lower() not in ['sign', 'sign_type'] and params[key]])


def _wechatpay_verify_notify(params):
    wechat_sign = params['sign']
    log.debug(u'wechat_sign = %s' % wechat_sign)
    filterParams = para_filter(params)
    filterParams['sign_type'] = 'MD5'
    string1 = convert_params_to_str_in_order(filterParams)
    stringSignTemp = string1 + '&key=%s' % settings.WECHATPAY_PARTNERKEY
    log.debug(u'stringSignTemp = %s' % stringSignTemp)
    md5 = hashlib.md5()
    md5.update(stringSignTemp)
    sign = md5.hexdigest().upper()
    log.debug(u'sign = %s' % sign)
    return wechat_sign == sign


###############################################################################
###############################################################################
###############################################################################
# 公众号支付


def wechatpay_mp_get_info(
        planid, out_trade_no, client_ip, openid='',
        trade_type='JSAPI', product_id=None):
    """ 公众号支付
        http://pay.weixin.qq.com/wiki/doc/api/index.php?chapter=9_1
        openid, trade_type=JSAPI，此参数必传，用户在商户appid下的唯一标识
        product_id, trade_type=NATIVE, 此参数必传。此id为二维码中包含的商品ID，商户自行定义
    """
    if trade_type == 'JSAPI' and not openid:
        log.error(u'trade_type = %s, openid = %s' % (trade_type, openid))
        return None, 'need openid'
    if trade_type == 'NATIVE' and not product_id:
        log.error(
            u'trade_type = %s, product_id = %s' % (trade_type, product_id)
        )
        return None, 'need product_id'

    plan = ChargePlan.objects.get(pk=int(planid))

    data = {}
    data['appid'] = settings.WECHATPAY_MP_APPID
    data['body'] = plan.name
    data['mch_id'] = settings.WECHATPAY_MP_MCH_ID
    data['nonce_str'] = random_str(13)
    data['notify_url'] = settings.WECHATPAY_MP_NOTIFY_URL
    if openid:
        data['openid'] = openid
    data['out_trade_no'] = out_trade_no
    data['spbill_create_ip'] = client_ip
    data['total_fee'] = plan.value
    data['trade_type'] = trade_type
    if product_id:
        data['product_id'] = product_id

    sign = wechatpay_mp_sign(data)
    data['sign'] = sign
    log.debug('data = %s' % data)
    xml = dicttoxml(data)
    log.debug('xml = %s' % xml)

    url = 'https://api.mch.weixin.qq.com/pay/unifiedorder'
    try:
        resp = requests.post(url=url, data=xml, timeout=10, verify=False)
    except Exception, e:
        log.warning(e)
        return None, 'requests Exception'

    log.debug('rsp = %s' % resp.content)
    content = xmltodict.parse(resp.content)
    log.debug('content = %s' % content)
    b = json.dumps(content)
    content = json.loads(b)['xml']
    log.debug('content = %s' % content)
    if content['return_code'] != 'SUCCESS':
        return None, content['return_msg']
    if content['result_code'] != 'SUCCESS':
        return None, content['err_code_des']
    else:
        return content['prepay_id'], None


def wechatpay_mp_sign(data):

    string1 = convert_params_to_str_in_order(data)
    stringSignTemp = string1 + '&key=%s' % settings.WECHATPAY_MP_SECRET
    log.debug('stringSignTemp = %s' % stringSignTemp)
    md5 = hashlib.md5()
    md5.update(stringSignTemp)
    sign_str = md5.hexdigest().upper()
    log.debug(u'sign = %s' % sign_str)
    return sign_str


@csrf_exempt
def wechat_mp_pay_notify(request):
    """ 微信公众号，
    """
    rsp_fail = {'return_code': 'FAIL'}
    if request.method != 'POST':
        log.debug(dicttoxml(rsp_fail))
        logging.error('equest.method != "POST"')
        return HttpResponse(dicttoxml(rsp_fail))

    log.debug(u'request.body = %s' % request.body)

    req_dict = xmltodict.parse(request.body)
    b = json.dumps(req_dict)
    req_dict = json.loads(b)['xml']
    log.debug('req_dict = %s' % req_dict)

    transaction_id = req_dict['transaction_id']
    order_id = req_dict['out_trade_no']

    if not wechat_mp_pay_verify(req_dict):
        return HttpResponse(dicttoxml(rsp_fail))
    try:
        cache_key = 'wechat_mp_pay_nid_' + \
            hashlib.sha1(transaction_id).hexdigest()
        nid = cache.get(cache_key)
        if nid:
            log.info(u'duplicated notify, drop it')
            log.info(u'request.body = %s' % request.body)
            return HttpResponse(dicttoxml(rsp_fail))

        if Charge.recharge(req_dict, provider='wechatmppay'):
            cache.set(cache_key, order_id, 90000)  # notify_id 保存25小时。
            log.info(u'wechatpay callback success')
            return HttpResponse(dicttoxml(
                {'return_code': 'SUCCESS', 'return_msg': 'OK'}
            ))
    except Exception, e:
        log.error(e)

    log.info(u'not a valid callback, ignore')
    return HttpResponse(dicttoxml(rsp_fail))


def wechat_mp_pay_verify(data):
    sign = data['sign']
    tmp_dict = {}
    for key, value in data.iteritems():
        if value and key != 'sign':
            tmp_dict[key] = value
    my_sign = wechatpay_mp_sign(tmp_dict)
    log.debug('sign    = %s' % sign)
    log.debug('my_sign = %s' % my_sign)

    if sign == my_sign:
        return True
    else:
        return False



# encoding=utf-8


APPWALLLOG_MAPPING = {
    'waps': {'identity': ('adv_id', 'key'),
             'cost': 'points',
             'user_id': 'key',
             'product_id': 'adv_id',
             'product_name': 'ad_name'
             },
    'youmi_ios': {'identity': 'order',
                  'cost': 'points',
                  'user_id': 'user',
                  'product_id': 'adid',
                  'product_name': 'ad'
                  },
    'youmi_adr': {'identity': 'order',
                  'cost': 'points',
                  'user_id': 'user',
                  'product_id': 'ad',
                  'product_name': 'ad',
                  },
    'dianjoy_adr': {'identity': ('device_id', 'pack_name',
                                 'trade_type', 'task_id'),
                    'cost': 'currency',
                    'user_id': 'snuid',
                    'product_id': 'pack_name',
                    'product_name': 'ad_name',
                    },
    'qumi': {'identity': 'order',
             'cost': 'points',
             'user_id': 'user',
             'product_id': 'ad',
             'product_name': 'ad',
             },
    'qumi_adr': {'identity': 'order',
                 'cost': 'points',
                 'user_id': 'user',
                 'product_id': 'ad',
                 'product_name': 'ad',
                 },
    'domob_adr': {
        'identity': 'orderid',
        'cost': 'point',
        'user_id': 'user',
        'product_id': 'adid',
        'product_name': 'ad',
    },
    'domob_ios': {
        'identity': 'orderid',
        'cost': 'point',
        'user_id': 'user',
        'product_id': 'adid',
        'product_name': 'ad',
    }
}

# wechatpay 微信app sdk支付
# wechatmppay 微信公众号sdk支付
PAYMENT_MAPPING = {
    'alipay': {'account': 'buyer_id',
               'email': 'buyer_email',
               'value': 'total_fee',
               'transaction_id': 'trade_no',
               'order_id': 'out_trade_no',
               'status': 'trade_status'
               },
    'wechatpay': {'account': 'OpenId',
                  'email': 'OpenId',
                  'value': 'total_fee',
                  'transaction_id': 'transaction_id',
                  'order_id': 'out_trade_no',
                  'status': 'trade_state'
                  },
    'wechatmppay': {
        'account': 'openid',
        'email': 'openid',
        'value': 'total_fee',
        'transaction_id': 'transaction_id',
        'order_id': 'out_trade_no',
        'status': 'result_code'
    }
}
PAYMENT_FINISH = {
    'alipay': 'TRADE_FINISHED',
    'wechatpay': '0',
    'wechatmppay': 'SUCCESS'
}

# 支付宝 1total_fee等于1蚊
# 微信支付和微信公众号支付 1total_fee等于1分
PAYMENT_SCALE = {
    'alipay': 100,
    'wechatpay': 1,
    'wechatmppay': 1,
}

EXCHANGE_RATE = 25  # 一RMB对应金币数

ALIPAY_PUBLIC_KEY = (
    '-----BEGIN PUBLIC KEY-----\n'
    'MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQCnxj/9qwVfgoUh/y2W89L6BkRA'
    'FljhNhgPdyPuBV64bfQNN1PjbCzkIM6qRdKBoLPXmKKMiFYnkd6rAoprih3/PrQE'
    'B/VsW8OoM8fxn67UDYuyBTqA23MML9q1+ilIZwBC2AQ2UBVOrFXfFl75p6/B5Ksi'
    'NG9zpgmLCUYuLkxpLQIDAQAB'
    '\n-----END PUBLIC KEY-----'
)



#!/usr/bin/env python

from setuptools import setup, find_packages
from goldencage import VERSION

url = "https://github.com/jalu017/perakcage"

long_description = "virtual coin & task management for mobile app (specially for china)"

setup(
    name="perakencage",
    version=VERSION,
    description=long_description,
    maintainer="jalu 017",
    maintainer_email="bebeknganjian@gmail.com",
    url=url,
    long_description=long_description,
    packages=find_packages('.'),
    zip_safe=False,
    install_requires=[
        'requests',
        'youhat',
        'pycrypto',
        'jsonfield',
        'pytz',
        'simplejson',
        'dicttoxml',
        'xmltodict',
        'youchat',
    ]
)
